<?php
require_once('common.php');

if(!authorized()){
    return;
}

if($_SERVER['REQUEST_METHOD'] != 'POST'){
    error_exit(200, "PROC: Request method not POST"); 
}

if(!isset($_POST["action"])){
    error_exit(201, "PROC: No action provided"); 
}

$action = $_POST['action'];

switch($action){
    case 'getUploads':
        getUploads();
        break;
    case 'deleteFile':
        deleteFile();
        break;
    case 'deleteSelected':
        deleteSelected();
        break;
    case 'zipSelected':
        zipSelected();
        break;
    case 'zipSelectedDownloads':
        zipSelectedDownloads();
        break;        
    case 'changePass':
        changePass();
        break;
    case 'copyToDownloads':
        copyToDownloads();
        break;
    case 'copySelectedToDownloads':
        copySelectedToDownloads();
        break;
    case 'deleteDownload':
        deleteDownload();
        break;
    case 'deleteSelectedDownloads':
        deleteSelectedDownloads();
        break;
    case 'getFileDownloads':
        getFileDownloads();
        break;
    case 'copyToUploads':
        copyToUploads();
        break;
    default:
        error_exit(202, "PROC: No valid action provided"); 
}

function changePass(){

    if(!isset($_POST['user'])){
        error_exit(203, "changePass: No username provided"); 
    }

    if(!isset($_POST['newpass'])){
        error_exit(204, "changePass: No new password provided"); 
    }
    
    if(!isset($_POST['newpass2'])){
        error_exit(205, "changePass: No confirmation password provided"); 
    }

    $user     = $_POST['user'];
    $oldpass  = trim($_POST['oldpass']);
    $newpass  = trim($_POST['newpass']);
    $newpass2 = trim($_POST['newpass2']);

    // This is just to double check.    
    if($user != "teacher"){
        error_exit(206, "changePass: wrong account name provided"); 
    }

    if($newpass != $newpass2){
        error_exit(207, "changePass: The new password does not match the confirmation password"); 
    }

    if(strlen($newpass) < 1 ){
        error_exit(208, "changePass: New password not provided"); 
    }
    
    if(strlen($newpass) < 6 ){
        error_exit(209, "changePass: New password must be 6 characters or more"); 
    }

    $db         = getdb();
    $db_oldpass = $db->escapeString(md5($oldpass));
    
    $validpass  = $db->querySingle(
        "SELECT 1 
           FROM users 
          WHERE username = 'teacher' 
            AND password = '$db_oldpass'");
    
    if (!$validpass) {
        error_exit(210, "changePass: Incorrect old password provided");
    }

    $db_newpass = $db->escapeString(md5($newpass));
    $result     = $db->exec("UPDATE users 
                                SET password = '$db_newpass' 
                              WHERE username = 'teacher'");
                          
    if(!$result){
        error_exit(211, "changePass: Failed to update password. " . $db->lastErrorMsg());
    }

    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "Password succesfully changed" ]);

    if(!$response){
        error_exit(212, "changePass: Failed to encode response");
    }

    echo $response;
    setcookie("teacher-auth", null, -1, "/");
    die();          
}

function copyToUploads(){
    if(!isset($_POST['name'])){
        error_exit(213, "copyToUploads: No file name provided"); 
    }
    
    $name         = $_POST['name'];
    $upload_dir   = dirname(__DIR__) . "/uploads/";
    $download_dir = dirname(__DIR__) . "/student_downloads/";
    $file_path    = $download_dir . $name;
    $save_path    = $upload_dir . $name;
    
    if(!file_exists($file_path)){
        error_exit(214, "copyToUploads: The provided file " . $file_path . " does not exist" );
    }

    if(file_exists($save_path)){
        error_exit(215, "copyToUploads: The provided file alreacy exists in the uploads directory" );
    }

    exec("cp '$file_path' '$upload_dir'", $output, $result);
    
    if($result == 1){
       error_exit(216, "copyToUploads: Failed to copy file " . $file_path ); 
    }
    
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "File is now downloadable" ]);

    if(!$response){
        error_exit(217, "copyToUploads: Failed to encode response");
    }

    echo $response;
    die();
}

function copyToDownloads(){
    if(!isset($_POST['name'])){
        error_exit(218, "copyToDownloads: No file name provided"); 
    }
    
    $name         = $_POST['name'];
    $upload_dir   = dirname(__DIR__) . "/uploads/";
    $download_dir = dirname(__DIR__) . "/student_downloads/";
    $file_path    = $upload_dir . $name;
    $save_path    = $download_dir . $name;
    
    if(file_exists($save_path)){
        error_exit(219, "copyToDownloads: A file with this name is already available for download." );
    }
    
    if(!file_exists($file_path)){
        error_exit(220, "copyToDownloads: The provided file " . $file_path . " does not exist" );
    }

    exec("cp '$file_path' '$download_dir'", $output, $result);
    
    if($result == 1){
       error_exit(221, "copyToDownloads: Failed to copy file " . $file_path ); 
    }
    
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "File is now downloadable" ]);

    if(!$response){
        error_exit(222, "copyToDownloads: Failed to encode response");
    }

    echo $response;
    die();
}

function copySelectedToDownloads(){
    if(!isset($_POST['selected'])){
        error_exit(223, "copySelectedToDownloads: No file list provided"); 
    }
    
    $json_files = json_decode($_POST['selected']);
    
    if($json_files == null){
        error_exit(224, "copySelectedToDownloads: Provided file list is null");         
    }

    $upload_dir   = dirname(__DIR__) . "/uploads/";
    $download_dir = dirname(__DIR__) . "/student_downloads/";

    $files = new RecursiveIteratorIterator(
        new RecursiveDirectoryIterator($upload_dir),
        RecursiveIteratorIterator::LEAVES_ONLY
    );

    foreach ($files as $name => $file){
        if ($file->isDir()){
            continue;
        }
        
        $exists = false;
        
        foreach($json_files as $json_file){
            if($json_file == basename($name)){
                $exists = true;
                break;
            }
        }
        
        if(!$exists){
            continue;
        }

        $filePath = $file->getRealPath();

        
        if(file_exists($download_dir . $name )){
            continue;
        }
        
        $result   = exec("cp '$filePath' '$download_dir'", $output, $result);

        if($result == 1){
            error_exit(225, "copySelectedToDownloads: Failed to copy file to downloads");
        }
    }

    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "Selected files are now downloadable" ]);

    if(!$response){
        error_exit(226, "copySelectedToDownloads: Failed to encode response");
    }

    echo $response;
    die();  
}

function getFileDownloads(){
    $upload_dir = dirname(__DIR__) . "/student_downloads/";
    $files      = scandir($upload_dir);
    $filenames  = array();

    foreach($files as $name){
        if($name == "." || $name == ".."){
            continue;
        }
        
        if($name == ".htaccess"){
            continue;
        }
        
        $file = $upload_dir. "/" . $name;
        

        array_push($filenames, array('name'     => $name,
                                     'size'     => formatBytes(filesize($file)),
                                     'changed'  => date ("F d Y H:i:s", filemtime($file))));	
    }

    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode($filenames);

    if(!$response){
        error_exit(227, "getUploads: Failed to encode response");
    }

    echo $response;
    die();
}

function getUploads(){
    $upload_dir = dirname(__DIR__) . "/uploads/";
    $files      = scandir($upload_dir);
    $filenames  = array();
    
    
    foreach($files as $name){
        if($name == "." || $name == ".."){
            continue;
        }
        
        if($name == ".htaccess"){
            continue;
        }
        
        $file = $upload_dir. "/" . $name;
        

        array_push($filenames, array('name'     => $name,
                                     'size'     => formatBytes(filesize($file)),
                                     'changed'  => date ("F d Y H:i:s", filemtime($file))));	
    }

    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode($filenames);

    if(!$response){
        error_exit(228, "getUploads: Failed to encode response");
    }

    echo $response;
    die();
}

function zipSelectedDownloads(){
    if(!isset($_POST['selected'])){
        error_exit(229, "zipSelectedDownloads: No file list provided"); 
    }
    
    $json_files = json_decode($_POST['selected']);
    
    if($json_files == null){
        error_exit(230, "zipSelectedDownloads: Provided file list is null");         
    }

    $rootPath   = dirname(__DIR__) . "/student_downloads/";
    $zipName    = $rootPath . "selected_downloads.zip";
    
    if(file_exists($zipName)){
        exec("rm '$zipName'", $output, $result);
    
        if($result == 1){
           error_exit(231, "zipSelectedDownloads: Failed to delete previous all_downloads.zip file" . $file_path ); 
        }
    }

    try {
        $zip = new ZipArchive();   
        $zip->open($zipName, ZipArchive::CREATE | ZipArchive::OVERWRITE);

        $files = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($rootPath),
            RecursiveIteratorIterator::LEAVES_ONLY
        );

        $count = 0;

        foreach ($files as $name => $file)    {
            if ($file->isDir()){
                continue;
            }
            
            $exists = false;
            
            foreach($json_files as $json_file){
                if($json_file == basename($name)){
                    
                    $exists = true;
                    break;
                }
            }
            
            if(!$exists){
                continue;
            }

            $filePath     = $file->getRealPath();
            $relativePath = substr($filePath, strlen($rootPath));
            $zip->addFile($filePath, $relativePath);
        }

        $zip->close();
    } catch( Exception $e){
        error_exit(232, "zipSelectedDownloads: Failed to zip files with error " );
    }
    
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "New selected_downloads.zip created" ]);

    if(!$response){
        error_exit(233, "zipSelectedDownloads: Failed to encode response");
    }

    echo $response;
    die();   
}

function zipSelected(){
    if(!isset($_POST['selected'])){
        error_exit(234, "zipSelected: No file list provided"); 
    }
    
    $json_files = json_decode($_POST['selected']);
    
    if($json_files == null){
        error_exit(235, "zipSelected: Provided file list is null");         
    }

    $rootPath   = dirname(__DIR__) . "/uploads/";
    $zipName    = $rootPath . "selected_uploads.zip";
    
    if(file_exists($zipName)){
        exec("rm '$zipName'", $output, $result);
    
        if($result == 1){
           error_exit(236, "zipSelected: Failed to delete previous allfiles.zip file" . $file_path ); 
        }
    }

    try {
        $zip = new ZipArchive();   
        $zip->open($zipName, ZipArchive::CREATE | ZipArchive::OVERWRITE);

        $files = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($rootPath),
            RecursiveIteratorIterator::LEAVES_ONLY
        );

        $count = 0;

        foreach ($files as $name => $file)    {
            if ($file->isDir()){
                continue;
            }
            
            $exists = false;
            
            foreach($json_files as $json_file){
                if($json_file == basename($name)){
                    
                    $exists = true;
                    break;
                }
            }
            
            if(!$exists){
                continue;
            }

            $filePath     = $file->getRealPath();
            $relativePath = substr($filePath, strlen($rootPath));
            $zip->addFile($filePath, $relativePath);
        }

        $zip->close();
    } catch( Exception $e){
        error_exit(237, "zipSelected: Failed to zip files with error " );
    }
    
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "New selected_uploads.zip created" ]);

    if(!$response){
        error_exit(238, "zipSelected: Failed to encode response");
    }

    echo $response;
    die();   
}

// Takes a list of files and deletes them
function deleteSelected(){
    if(!isset($_POST['selected'])){
        error_exit(239, "deleteSelected: No file list provided"); 
    }
    
    $json_files = json_decode($_POST['selected']);
    
    if($json_files == null){
        error_exit(240, "deleteSelected: Provided file list is null");         
    }

    $rootPath = dirname(__DIR__) . "/uploads/";
    

    $files = new RecursiveIteratorIterator(
        new RecursiveDirectoryIterator($rootPath),
        RecursiveIteratorIterator::LEAVES_ONLY
    );

    foreach ($files as $name => $file)    {
        if ($file->isDir()){
            continue;
        }
                       
        foreach($json_files as $json_file){
            if($json_file == basename($name)){
                exec("rm '$name'", $output, $result);
                
                if($result == 1){
                    error_exit(241, "deleteSelected: Failed to delete file " . basename($name) ); 
                }
                
                break;
            }
        }
    }
        
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "Selected files successfully deleted" ]);

    if(!$response){
        error_exit(242, "deleteSelected: Failed to encode response");
    }

    echo $response;
    die();   
}

// Delete the selected downloads
function deleteSelectedDownloads(){
    if(!isset($_POST['selected'])){
        error_exit(243, "downloadSelected: No file list provided"); 
    }
    
    $json_files = json_decode($_POST['selected']);
    
    if($json_files == null){
        error_exit(244, "downloadSelected: Provided file list is null");         
    }

    $rootPath = dirname(__DIR__) . "/student_downloads/";
    

    $files = new RecursiveIteratorIterator(
        new RecursiveDirectoryIterator($rootPath),
        RecursiveIteratorIterator::LEAVES_ONLY
    );

    foreach ($files as $name => $file)    {
        if ($file->isDir()){
            continue;
        }
                       
        foreach($json_files as $json_file){
            if($json_file == basename($name)){
                exec("rm '$name'", $output, $result);
                
                if($result == 1){
                    error_exit(245, "deleteSelected: Failed to delete file " . basename($name) ); 
                }
                
                break;
            }
        }
    }
        
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "Selected files successfully deleted" ]);

    if(!$response){
        error_exit(246, "deleteSelected: Failed to encode response");
    }

    echo $response;
    die();   
}

// Takes a single file name and deletes it 
function deleteDownload(){
    if(!isset($_POST['name'])){
        error_exit(247, "deleteFile: No file name provided"); 
    }
    
    $name = $_POST['name'];
    
    $file_path = dirname(__DIR__) . "/student_downloads/" . $name;
    
    if(!file_exists($file_path)){
        error_exit(248, "deleteFile: The provided file " . $file_path . " does not exist" );
    }
    
    exec("rm '$file_path'", $output, $result);
    
    if($result == 1){
       error_exit(249, "deleteFile: Failed to delete file " . $file_path ); 
    }
    
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "File successfully deleted" ]);

    if(!$response){
        error_exit(250, "deleteFile: Failed to encode response");
    }

    echo $response;
    die();
}

// Takes a single file name and deletes it 
function deleteFile(){
    if(!isset($_POST['name'])){
        error_exit(251, "deleteFile: No file name provided"); 
    }
    
    $name = $_POST['name'];
    
    $file_path = dirname(__DIR__) . "/uploads/" . $name;
    
    if(!file_exists($file_path)){
        error_exit(252, "deleteFile: The provided file " . $file_path . " does not exist" );
    }
    
    exec("rm '$file_path'", $output, $result);
    
    if($result == 1){
       error_exit(253, "deleteFile: Failed to delete file " . $file_path ); 
    }
    
    header("HTTP/1.1 200 OK");
    header('Content-Type: application/json; charset=UTF-8');

    $response = json_encode(["responseText" => "File successfully deleted" ]);

    if(!$response){
        error_exit(254, "deleteFile: Failed to encode response");
    }

    echo $response;
    die();
}

function formatBytes($size, $precision = 2){
    if($size == 0){
        return '0 kb';
    }
    
    $base     = log($size, 1024);
    $suffixes = array('', 'kb', 'mb', 'gb', 'tb');   

    return round(pow(1024, $base - floor($base)), $precision) .' '. $suffixes[floor($base)];
}


// Converts PHP values to bytes 
function PHPToBytes( $value ) {
    $len    = strlen($value) - 1;
    $size   = substr( $value, 0, $len);
    $letter = strtoupper( substr( $value, $len));
    
    if($letter == "K"){
        return $size * 1024;
    }
    
    if($letter == "M"){
        return $size * 1048576;
    }
    
    if($letter == "G"){
        return $size * 1073741824;
    }
    
    error_exit(255, "PHPToBytes: Failed to get letter from string");
}


?>