<?php

require_once('common.php');

if(!authorized()){
    return;
}

?>

<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <title>Turn it in - Student Submissions</title>
  <meta name="description" content="A module for submitting homework">
  <meta name="author"      content="World Possible">
  <meta name="viewport"    content="width=device-width, initial-scale=1">
  <link rel="stylesheet" href="../assets/css/normalize.css">
  <link rel="stylesheet" href="../assets/css/skeleton.css">
  <link rel="stylesheet" href="../assets/css/custom.css">
  <link rel="stylesheet" href="../assets/fontawesome/css/all.css">
  <link rel="icon" type="image/png" href="../assets/images/favicon.png">
  <script type="text/javascript" src="../assets/js/jquery-3.6.0.min.js"></script>
  <script>
    
  var lastSelect = true;
  
  function selectAll(){
      var checkboxes = document.getElementsByName('file_select');
     
      for(var i=0; i < checkboxes.length; i++) {
          checkboxes[i].checked = lastSelect;
      }
      
      lastSelect = !lastSelect;
  }
  
  function zipSelected(){      
      var checkboxes   = document.getElementsByName('file_select');
      var files = [];
     
      for(var i=0; i < checkboxes.length; i++) {
          if(checkboxes[i].checked){
              var file = checkboxes[i].value;
              
              if(file == "selected_uploads.zip"){
                  
                  continue;
              }
              
              files.push(file);
          }
      }
      
      if(files.length == 0){
          alert('No files selected to zip. ');
          return;
      }
      
      var selectedFiles = JSON.stringify(files);

      $("#zip").replaceWith("<img src='../assets/images/spinner.gif'>");
      
      // show spinner on the button itself

      $.ajax({
        url: "process.php",
        data: { action: 'zipSelected', "selected": selectedFiles },
        type: "POST",
        dataType : "json",
        success: function(data) {
            alert(data['responseText']);
            location.reload();
        },
        error: function(response) {
            alert(response['responseText']);
        }
      });
  }
  
  function deleteSelected(){
      var checkboxes   = document.getElementsByName('file_select');
      var checkedfiles = [];
     
      for(var i=0; i < checkboxes.length; i++) {
          if(checkboxes[i].checked){
              var file = checkboxes[i].value;
              checkedfiles.push(file);
          }
      }
      
      if(checkedfiles.length == 0){
          return;
      }

      if (!confirm('Are you sure you want to delete all of the selected files?')) {
          return;
      } 

      var selectedFiles = JSON.stringify(checkedfiles);
      
      $.ajax({
        url: "process.php",
        data: { action: 'deleteSelected', "selected": selectedFiles },
        type: "POST",
        dataType : "json",
        success: function(data) {
            alert("Files successfully deleted");
            location.reload();
        },
        error: function(response) {
            alert(response['responseText']);
        }
      });
  }
    
  function deleteFile(name){
      $.ajax({
        url: "process.php",
        data: { action: 'deleteFile', "name": name },
        type: "POST",
        success: function(data) {
            alert("The file was successfully deleted");
            location.reload();
        },
        error: function(response) {
            alert(response['responseText']);
        }
      });      
  }
  
  function getFileList(){

    // Remove everything in the table
    $("#file_list").find("tr:gt(1)").remove();

    $.ajax({
        url: "process.php",
        data: { action: 'getUploads' },
        type: "POST",
        dataType : "json",
        success: function(data) {

            data.sort( function( a, b ) {
                a = a['name'].toLowerCase();
                b = b['name'].toLowerCase();

                return a < b ? -1 : a > b ? 1 : 0;
            });

            for(var i = 0; i < data.length; i++){
                var name   = data[i]['name'];
                var path   = "../uploads/" + name;
                var table  = "<tr>";
                table     += "<td><input type='checkbox' id='' name='file_select' value=\"" + name + "\"></td>"
                table     += "<td><a href=\"" + path + "\" >" + name + "</a></td>"; 
                table     += "<td>" + data[i]['size'] + "</td>";
                table     += "<td>" + data[i]['changed'] + "</td>";
                table     += "<td colspan='3'><a title='Download File' class='button-2 small' href=\"" + path + "\" download=\"" + name + "\"><i class='fas fa-cloud-download-alt'></i></a>";
                
                table     += " <button title='Delete File' class='button-2 small' onclick='deleteFile(\"" + name + "\");'><i class='far fa-trash-alt'></i></button>";
               
                $('#file_list tr:last').after(table);
            }

            if(data.length > 0){
                // Add the "All Files" seection
                var table  = "<tr><td><input type='checkbox' id='file_all' name='file_select' onchange='selectAll()'></td>";         
                table     += "<td>All Files </td><td>&nbsp;</td><td>&nbsp;</td>";
                table     += "<td> <button id='zip' title='Zip Selected files' class='button-2 small' onclick='zipSelected();'><i class='far fa-file-archive'></i></button>";
                table     += " <button title='Delete Selected'  class='button-2 small' onclick='deleteSelected();'><i class='far fa-trash-alt'></i></button>";
                $('#file_list tr:last').after(table);
            }

           if(data.length == 0){
                $('#file_list tr:last').after('<tr><td>No files available</td></tr>');
           }

        },
        error: function(response) {
            $('#file_list tr:last').after('<tr><td>Error: ' +  response.responseJSON['responseText'] + '</td></tr>');
        }
    });
  }
  
  $(function() {
    getFileList();
  });
  
  </script>

  
</head>
<body>
<?php include('navbar.php'); ?>

  <div class="section u-pull-left">
    <div class="container">     
      <h3 class="section-heading u-pull-left">Student Submissions</h3>
      <p class="section-description u-pull-left cf">The following files were uploaded via the main "turn it in" page</p>      
      
      <table id="file_list">
         <th> </th>
         <th>Name</th>
         <th>Size</th>
         <th>Last Modified</th>
      </table>
      <ul>
        <li class="u-pull-left wrap">To download all files, first select check box next to "All Files" and select zip icon. Then download the zipped file after it is created.This may take some time for large files</li>
        <li class="u-pull-left wrap">The "selected_uploads.zip" file will not be included when zipping selected uploads
        </li>
      </ul>
  </div>
  </div>
</body>
</html>
